clear variables
format long


%=======================================================================
%This code is associated with the paper 
%
%Computer assisted proofs for transverse collision and near collision
%orbits in the restricted three body problems
%
%By: Maciej Capinski, Shane Kepley, and J.D. Mireles James
%
%All references to ``the paper'' refer to the title above
%
%
%======================================================================

%*************************************************************************
%This program locates a good initial guess for the data needed in Theorem 1
%*************************************************************************
%
%The parameters are:

mu = 1/4
m1 = 1-mu    
m2 = mu

C = 3.2

%Then we compute the radi for the collision circles in 
% the m2 and m1 regularizations respectively:
%
R0 = sqrt(2*crtbp_regularized_U(0, 0, C, mu))
R1 = sqrt(2*crtbp_regularized_U_m1(0, 0, C, mu))

%Comute the Hill's regions for this energy level:
[X_grid,Y_grid]=meshgrid(-2.2:0.01:2.2);
    R1_grid=((X_grid - m2).^2 + Y_grid.^2).^(1/2);
    R2_grid=((X_grid + m1).^2 + Y_grid.^2).^(1/2);
    Z_grid=[m1*(0.5*R1_grid.^2 + R1_grid.^(-1))+...
        m2*(0.5*R2_grid.^2 + R2_grid.^(-1))];
   
%Plot the Hill's regions:     
figure
hold on
plot3(-m1, 0, 0, 'k*')
plot3(m2, 0, 0, 'k*')
contour(X_grid,Y_grid,2*Z_grid,[C,C],'r')





numPoints = 1000
thetas = linspace(0, 2*pi, numPoints);
collisions = zeros(4, numPoints);
for n = 1:numPoints
   thisTheta = thetas(n);
   thisPoint = [                0;
                R0*cos(thisTheta);
                                0;
                R0*sin(thisTheta)];
   collisions(1:4, n) = thisPoint;         
end


orbitPoints = 25;
collisionManifold = [];
collisionManifold_back = [];
collisionManifold_home = [];
collisionManifold_home_back = [];
collisionBoundary1 = zeros(4, numPoints);
collisionBoundary2 = zeros(4, numPoints);


T = 0.35

for n = 1:numPoints
    n
    tspan = linspace(0, T, orbitPoints);
    tspan_minus = linspace(0, -T, orbitPoints);
    IC = collisions(1:4, n);

    %Integraton:
    options=odeset('RelTol',1e-13,'AbsTol',1e-13);
    [t,thisOrbit] = ode15s('crtb_regularizedField', ...
                           tspan, IC, options, flag, mu, C);                  
                      
    [t,thisOrbit_back] = ode15s('crtb_regularizedField', ...
                           tspan_minus, IC, options, flag, mu, C);                   
                 
    collisionManifold = [collisionManifold, thisOrbit'];
    collisionManifold_back = [collisionManifold_back, thisOrbit_back'];
    collisionBoundary1(1:4, n) = thisOrbit(end, :)';
    collisionBoundary2(1:4, n) = thisOrbit_back(end, :)';
end

 
transformedBoundary1 = returnToCRTBP(collisionBoundary1', mu, C);
transformedBoundary2 = returnToCRTBP(collisionBoundary2', mu, C);






[X_grid,Y_grid]=meshgrid(-2.2:0.001:2);
    R1_grid=((X_grid - m2).^2 + Y_grid.^2).^(1/2);
    R2_grid=((X_grid + m1).^2 + Y_grid.^2).^(1/2);
    Z_grid=[m1*(0.5*R1_grid.^2 + R1_grid.^(-1))+...
        m2*(0.5*R2_grid.^2 + R2_grid.^(-1))];
   

   
    
figure
hold on
plot3(transformedBoundary1(1, :), ...
      transformedBoundary1(3, :), ...
      transformedBoundary1(2, :), 'r.')

plot3(transformedBoundary2(1, :), ...
      transformedBoundary2(3, :), ...
      transformedBoundary2(2, :), 'b.')

plot3(-m1, 0, 0, 'k*')
plot3(m2, 0, 0, 'k*')
contour(X_grid,Y_grid,2*Z_grid,[C,C],'r')



numPoints = 100
thetas = linspace(0, 2*pi, numPoints);
collisions = zeros(4, numPoints);
for n = 1:numPoints
   thisTheta = thetas(n);
   thisPoint = [                0;
                R1*cos(thisTheta);
                                0;
                R1*sin(thisTheta)];
   collisions(1:4, n) = thisPoint;         
end



orbitPoints = 25;
collisionManifold = [];
collisionManifold_back = [];
collisionManifold_home = [];
collisionManifold_home_back = [];
collisionBoundary1 = zeros(4, numPoints);
collisionBoundary2 = zeros(4, numPoints);


T = 0.35

for n = 1:numPoints
    n
    tspan = linspace(0, T, orbitPoints);
    tspan_minus = linspace(0, -T, orbitPoints);
    IC = collisions(1:4, n);

    %Integraton:
    options=odeset('RelTol',1e-13,'AbsTol',1e-13);
    [t,thisOrbit] = ode45('crtb_regularizedField_mu1', ...
                           tspan, IC, options, flag, 1-mu, C);                  
                      
    [t,thisOrbit_back] = ode45('crtb_regularizedField_mu1', ...
                           tspan_minus, IC, options, flag, 1-mu, C);                   
                 
    collisionManifold = [collisionManifold, thisOrbit'];
    collisionManifold_back = [collisionManifold_back, thisOrbit_back'];
    collisionBoundary1(1:4, n) = thisOrbit(end, :)';
    collisionBoundary2(1:4, n) = thisOrbit_back(end, :)';
end

 
transformedBoundary3 = returnToCRTBP_mu1(collisionBoundary1', mu, C);
transformedBoundary4 = returnToCRTBP_mu1(collisionBoundary2', mu, C);

    
figure
hold on

plot3(transformedBoundary1(1, :), ...
      transformedBoundary1(3, :), ...
      transformedBoundary1(2, :), 'b', 'LineWidth', 3)

% plot3(transformedBoundary2(1, :), ...
%       transformedBoundary2(3, :), ...
%       transformedBoundary2(2, :), 'c.')
% 
% 
% plot3(transformedBoundary3(1, :), ...
%       transformedBoundary3(3, :), ...
%       transformedBoundary3(2, :), 'g.')

plot3(transformedBoundary4(1, :), ...
      transformedBoundary4(3, :), ...
      transformedBoundary4(2, :), 'c', 'LineWidth', 3)

  
 
plot3(-m1, 0, 0, 'k*')
plot3(m2, 0, 0, 'k*')
contour(X_grid,Y_grid,2*Z_grid,[C,C],'r', 'LineWidth', 4)
 
  
 

% ======================================================================
%
% Find collisions
%
% ======================================================================


%RECALL:
% R0 = sqrt(2*crtbp_regularized_U(0, 0, C, mu))      -- Large body 
% R1 = sqrt(2*crtbp_regularized_U_m1(0, 0, C, mu))   -- Secondary body



numPoints = 20
thetas = linspace(2.945, 2.946, numPoints);


%Between 2.1 and 2.6 - problematic... goes through self collision first...
%Maybe near   2.95, 

%For the smaller body:
collisions = zeros(4, numPoints);
for n = 1:numPoints
   thisTheta = thetas(n);
   thisPoint = [                0;
                R1*cos(thisTheta);
                                0;
                R1*sin(thisTheta)];
   collisions(1:4, n) = thisPoint;         
end


orbitPoints = 25;
collisionManifold = [];
collisionManifold_back = [];
collisionManifold_home = [];
collisionManifold_home_back = [];
collisionBoundary1 = zeros(4, numPoints);

T = 0.35

%Flow in samll body regular coordinates:
for n = 1:numPoints
    n
    tspan = linspace(0, T, orbitPoints);
    IC = collisions(1:4, n);

    %Integraton:
    options=odeset('RelTol',1e-13,'AbsTol',1e-13);
    [t, thisOrbit] = ode45('crtb_regularizedField_mu1', ...
                           tspan, IC, options, flag, 1-mu, C);                  
                                     
    collisionManifold = [collisionManifold, thisOrbit'];
    collisionBoundary1(1:4, n) = thisOrbit(end, :)';
    
end

transformedBoundary3 = returnToCRTBP_mu1(collisionBoundary1', mu, C);

T = 4
distances = []; 

for n = 1:numPoints
    
    n
   T = 2.052
   n = 11
   IC = transformedBoundary3(1:4, n)';
   tspan = linspace(0, T, 500);
    %Integraton:

    options=odeset('RelTol',1e-13,'AbsTol',1e-13);
    [t, orbit] = ode15s('crtb_Field', ...
         tspan, IC, options, flag, m1, m2);
    
     
    d = set2setDistance(orbit', transformedBoundary2) 
    distances = [distances, d];
     
figure
hold on

plot3(orbit(:, 1), ...
      orbit(:, 3), ...
      orbit(:, 2), 'b.')

 
plot3(transformedBoundary2(1, :), ...
      transformedBoundary2(3, :), ...
      transformedBoundary2(2, :), 'k.')


plot3(transformedBoundary3(1, :), ...
      transformedBoundary3(3, :), ...
      transformedBoundary3(2, :), 'g.')


plot(-m1, 0, 'k*')
plot(m2, 0, 'k*')
%contour(X_grid,Y_grid,2*Z_grid,[C,C],'r')
      
end


figure 
hold on 
plot(distances, 'b')



%Find angle on the other side:

%theta = 0.829736842105263
%theta = 0.826157894736842

theta = 2.945526315789474,  T = 2.052

numPoints2 = 1000

bestIndex = 0;
thisPoint = orbit(end, :)';
bestDist = 100;
sigmas = linspace(0, 2*pi, numPoints2);
bestSigma = 7;
T2 = -0.35 

for n = 1:numPoints2
    
   sigma = sigmas(n);
   thisColision = [ 0;
        R0*cos(sigma);
                    0;
        R0*sin(sigma)];
    
       
    [phiP, DphiP] = time_T_flow_C1_regularized(...
        thisColision(1), ...
        thisColision(2), ...
        thisColision(3), ...
        thisColision(4), ...
        T2, mu, C)

    [x2, u2, y2 , v2] = ...
      regularCoordinatesInverse(...
        phiP(1), phiP(2), phiP(3), phiP(4), ...
        mu, C)

      
    endPoint = [x2, u2, y2 , v2]'
    d = norm(endPoint - thisPoint, inf)
    
    if d < bestDist
        bestDist = d
        bestIndex = n
        bestSigma = sigma
    end
    
end
  

bestSigma




col1 = [            0;
        R0*cos(bestSigma);
                    0;
        R0*sin(bestSigma)];
  
   
[phiP, DphiP] = time_T_flow_C1_regularized(...
    col1(1), col1(2), col1(3), col1(4), T2, mu, C)

[x2, u2, y2 , v2] = ...
    regularCoordinatesInverse(...
        phiP(1), phiP(2), phiP(3), phiP(4), mu, C)

      
endPoint = [x2, u2, y2 , v2]'
    
check_end = norm(thisPoint - endPoint, inf)  


  
col0 = [            0;
        R1*cos(theta);
                    0;
        R1*sin(theta)];
     
check0 = norm(col0 - collisions(1:4, 15), inf)  
    
T1 = 0.35    
[phiS, DphiS] = time_T_flow_C1_regularized_m1(...
    col0(1), col0(2), col0(3), col0(4), T1, m1, C)
       

[x1, u1, y1 , v1] = ...
    regularCoordinatesInverse_mu1(...
        phiS(1), phiS(2), phiS(3), phiS(4), m1, C)

IC0 = [x1, u1, y1 , v1]'   
    

check1 = norm(IC0 - transformedBoundary3(1:4, 5), inf)


alpha = 0.0


[phi, Dphi, partial_alpha_phi] = ...
    time_T_flow_PCRTBP_unfolding(...
    IC0(1), IC0(2), IC0(3), IC0(4),T,alpha,mu)
          
    
errorTest = norm(phi - orbit(end, :)', inf)
    

trueErrorTest = norm(endPoint - phi, inf)

sigma = bestSigma
tau = T1
x1 = col0
x2 = phiS
x3 = IC0
x4 = phi
x5 = phiP

[G, DG] = primaryToSecondary_collisionMap(...
    theta, sigma, T, alpha, x1, x2, x3, x4, x5, mu, C, tau);


for k = 1:6
    
    State = [theta;
             sigma;
                 T;
             alpha;
                x1;
                x2;
                x3;
                x4;
                x5];
            
    State = State - DG\G;
    
    theta = State(1);
    sigma = State(2);
    T = State(3);
    alpha = State(4);
    x1 = State(5:8);
    x2 = State(9:12);
    x3 = State(13:16);
    x4 = State(17:20);
    x5 = State(21:24);
    
    [G, DG] = primaryToSecondary_collisionMap(...
    theta, sigma, T, alpha, x1, x2, x3, x4, x5, mu, C, tau);

    thisError = norm(DG\G, inf)
end

theta
x1
x2
x3
x4
x5
sigma
T

