function cubic_hat = fast_cubic_hat(a,b,c,m,n)
%FAST_CUBIC_HAT - Fast computation of a(hat)b(hat)c using Cauch products
%   Syntax:
%       output = FAST_CUBIC_HAT(input1, input2)
%       [output1, output2] = FAST_CUBIC_HAT(input1, input2, input3)
%
%   Inputs:
%       input1 - Description
%       input2 - Description
%       input3 - Description
%
%   Outputs:
%       output1 - Description
%       output2 - Description
%
%   Subfunctions: none
%   Classes required: none
%   Other m-files required: none
%   MAT-files required: none

%   Author: Shane Kepley
%   email: shane.kepley@rutgers.edu
%   Date: 18-Mar-2019; Last revision: 18-Mar-2019

A = a(1:m+1,1:n+1);
B = b(1:m+1,1:n+1);
C = c(1:m+1,1:n+1);
A(m+1,n+1) = intval(0);
B(m+1,n+1) = intval(0);
C(m+1,n+1) = intval(0);
AB = intvaltimes(A,B,1:m+1,1:n+1); % truncated Cauchy product with all lower order terms
cubic_hat = intvaltimes(AB,C,m+1,n+1); % return (m,n) term of last Cauchy product with
end % end fast_cubic_hat

% Revision History:
%{

%}
