function [G, DG] = primaryCollisionToL4_connectionMap_multShooting(...
    theta, sigma, T, alpha, X, Y, K1, K2, ...
    As, Bs, Cs, Ds, N, mu, C, R, tau)

%X is a matrix with 4 rows and (K1+1) columns
%Y is a matrix with 4 rows and (K2+1) columns
%Z is a matrix with 4 rows and  K3*4  columns
%
%The system has K1+K2+K3+3 4-d rows.

Id = eye(4);
ZeroMat = zeros(4, 4);
ZeroCol = zeros(4, 1);

[SP_theta, DSP_theta] = collisionSet_PrimaryBody(theta, mu, C);
G = SP_theta - X(:, 1);


index1 = 0;
index2 = K1 + K2;
D1 = [];


for k = 1:K1

       [phi_Pxk, Dphi_Pxk] = ...
              time_T_flow_C1_regularized(...
                    X(1,k), X(2,k), X(3, k), X(4, k), tau/K1, mu, C); 
                
       G = [G; phi_Pxk - X(:, k+1)];  
       
       
       temp_col = [];
       for m = 1:index1
           temp_col = [temp_col; 
                        ZeroMat];
       end

       temp_col = [temp_col; 
                        -Id;
         Dphi_Pxk(1:4, 1:4)];
       
       for n = 1:index2;
           temp_col = [temp_col;
                        ZeroMat];
       end
       
       
       D1 = [D1, temp_col];
       
       index1 = index1 + 1;
       index2 = index2 - 1;
end

[TPR_xK1_plus_1, DTPR_xK1_plus_1] = ...
    transform_primaryToRotating(X(:, K1+1), mu, C); 

G = [G; TPR_xK1_plus_1 - Y(:, 1)];

temp_col = [];
for m = 1:index1
    temp_col = [temp_col; 
                ZeroMat];
end

temp_col = [temp_col; 
                 -Id;
     DTPR_xK1_plus_1];
       
for n = 1:index2;
    temp_col = [temp_col;
                 ZeroMat];
end
       
       
D1 = [D1, temp_col];
       
index1 = index1 + 1;
index2 = index2 - 1;

D_T = [];
D_alpha = [];

for k = 1:K2-1
    
    [phi_Ryk, Dphi_Ryk, partial_alpha_phi_Ryk] = ...
         time_T_flow_PCRTBP_unfolding(...
             Y(1,k), Y(2,k), Y(3,k), Y(4,k), T/K2, alpha, mu);
         
    G = [G; phi_Ryk - Y(:, k+1)];  
    
    D_T = [D_T; Dphi_Ryk(1:4, 5)/K2];
    D_alpha = [D_alpha; partial_alpha_phi_Ryk];
    
    temp_col = [];
    for m = 1:index1
        temp_col = [temp_col; 
                    ZeroMat];
    end

    temp_col = [temp_col; 
                     -Id;
    Dphi_Ryk(1:4, 1:4)];
       
    for n = 1:index2;
        temp_col = [temp_col;
                     ZeroMat];
    end
          
    D1 = [D1, temp_col];
       
    index1 = index1 + 1;
    index2 = index2 - 1;

end


[P_sigma, DP_sigma] = parmBD_L4_saddleFocus(...
                sigma, R, As, Bs, Cs, Ds, N);

 [phi_RyK2, Dphi_RyK2, partial_alpha_phi_RyK2] = ...
         time_T_flow_PCRTBP_unfolding(...
             Y(1,K2), Y(2,K2), Y(3,K2), Y(4,K2), T/K2, alpha, mu);
      
D_alpha = [D_alpha; partial_alpha_phi_RyK2];
D_T = [D_T; Dphi_RyK2(1:4, 5)/K2];         
G = [G; phi_RyK2 - P_sigma];

temp_col = [];
for m = 1:index1
    temp_col = [temp_col; 
                 ZeroMat];
end

temp_col = [temp_col; 
                 -Id;
  Dphi_RyK2(1:4, 1:4)];
       
for n = 1:index2;
    temp_col = [temp_col;
                 ZeroMat];
end
          
D1 = [D1, temp_col];
D01 = DSP_theta;

for m = 1:K1+K2+1
    D01 = [D01;
       ZeroCol];
end

D02 = [];
for m = 1:K1+K2+1
    D02 = [D02;
       ZeroCol];
end

D02 =     [D02;
    -DP_sigma];

D03 = [];
D04 = [];
for m = 1:K1+2
    D03 = [D03;
       ZeroCol];
   
   D04 = [D04;
      ZeroCol];
end

D03 = [D03; D_T];
D04 = [D04; D_alpha];


DG = [D01, D02, D03, D04, D1];

end

