function [G, DG] = primaryToSecondary_collisionMap_multShootingII(...
    theta, sigma, T, alpha, X, Y, Z, K1, K2, K3, mu, C, tau)

%X is a matrix with 4 rows and (K1+1) columns
%Y is a matrix with 4 rows and (K2+1) columns
%Z is a matrix with 4 rows and  K3*4  columns
%
%The system has K1+K2+K3+3 4-d rows.

Id = eye(4);
ZeroMat = zeros(4, 4);
ZeroCol = zeros(4, 1);

[SS_theta, DSS_theta] = collisionSet_PrimaryBody(theta, mu, C);
G = SS_theta - X(:, 1);


index1 = 0;
index2 = K1 + K2 + K3 + 1;
D1 = [];


for k = 1:K1

       [phi_Sxk, Dphi_Sxk] = ...
              time_T_flow_C1_regularized(...
                    X(1,k), X(2,k), X(3, k), X(4, k), ...
                    tau/K1, mu, C); 
                
       G = [G; phi_Sxk - X(:, k+1)];  
       
       
       temp_col = [];
       for m = 1:index1
           temp_col = [temp_col; 
                        ZeroMat];
       end

       temp_col = [temp_col; 
                        -Id;
         Dphi_Sxk(1:4, 1:4)];
       
       for n = 1:index2;
           temp_col = [temp_col;
                        ZeroMat];
       end
       
       
       D1 = [D1, temp_col];
       
       index1 = index1 + 1;
       index2 = index2 - 1;
end

[TSR_xK1_plus_1, DTSR_xK1_plus_1] = ...
    transform_primaryToRotating(X(:, K1+1), mu, C); 

G = [G; TSR_xK1_plus_1 - Y(:, 1)];

temp_col = [];
for m = 1:index1
    temp_col = [temp_col; 
                ZeroMat];
end

temp_col = [temp_col; 
                 -Id;
     DTSR_xK1_plus_1];
       
for n = 1:index2;
    temp_col = [temp_col;
                 ZeroMat];
end
       
       
D1 = [D1, temp_col];
       
index1 = index1 + 1;
index2 = index2 - 1;

D_T = [];
D_alpha = [];

for k = 1:K2
    
    [phi_Ryk, Dphi_Ryk, partial_alpha_phi_Ryk] = ...
         time_T_flow_PCRTBP_unfolding(...
             Y(1,k), Y(2,k), Y(3,k), Y(4,k), T/K2, alpha, mu);
         
    G = [G; phi_Ryk - Y(:, k+1)];  
    
    D_T = [D_T; Dphi_Ryk(1:4, 5)/K2];
    D_alpha = [D_alpha; partial_alpha_phi_Ryk];
    
    temp_col = [];
    for m = 1:index1
        temp_col = [temp_col; 
                    ZeroMat];
    end

    temp_col = [temp_col; 
                     -Id;
      Dphi_Ryk(1:4, 1:4)];
       
    for n = 1:index2;
        temp_col = [temp_col;
                     ZeroMat];
    end
          
    D1 = [D1, temp_col];
       
    index1 = index1 + 1;
    index2 = index2 - 1;

end

[TRP_yK2_plus_1, DTRP_yK2_plus_1] = ...
    transform_rotatingToSecondary(Y(:, K2+1), mu, C);  

G = [G; TRP_yK2_plus_1 - Z(:, 1)];

temp_col = [];
for m = 1:index1
    temp_col = [temp_col; 
                ZeroMat];
end

temp_col = [temp_col; 
                 -Id;
     DTRP_yK2_plus_1];
       
for n = 1:index2;
    temp_col = [temp_col;
                 ZeroMat];
end
          
D1 = [D1, temp_col];
       
index1 = index1 + 1;
index2 = index2 - 1;

for k = 1:K3-1
    
   [phi_Pzk, Dphi_Pzk] = time_T_flow_C1_regularized_m1(...
      Z(1,k), Z(2,k), Z(3,k), Z(4,k), tau/K3, 1-mu, C);
  
   G = [G; phi_Pzk - Z(:, k+1)];
   
   temp_col = [];
   for m = 1:index1
       temp_col = [temp_col; 
                    ZeroMat];
   end

   temp_col = [temp_col; 
                    -Id;
     Dphi_Pzk(1:4, 1:4)];
       
   for n = 1:index2;
       temp_col = [temp_col;
                    ZeroMat];
   end
          
   D1 = [D1, temp_col];
       
   index1 = index1 + 1;
   index2 = index2 - 1;
    
end

[SP_sigma, DSP_sigma] = collisionSet_secondaryBody(sigma, mu, C);
[phi_PzK3, Dphi_PzK3] = time_T_flow_C1_regularized_m1(...
      Z(1,K3), Z(2,K3), Z(3,K3), Z(4,K3), tau/K3, 1-mu, C);

G = [G; phi_PzK3 - SP_sigma];


temp_col = [];
for m = 1:index1
    temp_col = [temp_col; 
                 ZeroMat];
end

temp_col = [temp_col; 
                 -Id;
  Dphi_PzK3(1:4, 1:4)];
       
for n = 1:index2;
    temp_col = [temp_col;
                 ZeroMat];
end
          
D1 = [D1, temp_col];

D01 = DSS_theta;

for m = 1:K1+K2+K3+2
    D01 = [D01;
       ZeroCol];
end

D02 = [];
for m = 1:K1+K2+K3+2
    D02 = [D02;
       ZeroCol];
end

D02 =     [D02;
    -DSP_sigma];

D03 = [];
D04 = [];
for m = 1:K1+2
    D03 = [D03;
       ZeroCol];
   
   D04 = [D04;
      ZeroCol];
end

D03 = [D03; D_T];
D04 = [D04; D_alpha];

for m = 1:K3+1
    D03 = [D03;
       ZeroCol];
   
   D04 = [D04;
      ZeroCol];
end

DG = [D01, D02, D03, D04, D1];

end




% 
% [phi_Sx1, Dphi_Sx1] = ...
%     time_T_flow_C1_regularized_m1(...
%        x1(1), x1(2), x1(3), x1(4), tau, 1-mu, C);
%     
% [phi_Rx3, Dphi_Rx3, partial_alpha_phi] = ...
%     time_T_flow_PCRTBP_unfolding(...
%        x3(1), x3(2), x3(3), x3(4), T, alpha, mu);
% 
%  FPhi_Rx3 = crtbField_unfolding(phi_Rx3, mu, alpha);   
%    
% [phi_Px5, Dphi_Px5] = time_T_flow_C1_regularized(...
%     x5(1), x5(2), x5(3), x5(4), tau, mu, C);
%      
% [SS_theta, DSS_theta] = collisionSet_secondaryBody(theta, mu, C);
% [SP_sigma, DSP_sigma] = collisionSet_PrimaryBody(sigma, mu, C);
% 
% [TSR_x2, DTSR_x2] = transform_secondaryToRotating(x2, mu, C); 
% [TRP_x4, DTRP_x4] = transform_rotatingToPrimary(x4, mu, C);          
% 
%            
% G1 = SS_theta - x1;
% G2 = phi_Sx1 - x2;
% G3 = TSR_x2 - x3;
% G4 = phi_Rx3 - x4;
% G5 = TRP_x4 - x5;
% G6 = phi_Px5 - SP_sigma;
% 
% G = [G1;
%      G2;
%      G3;
%      G4; 
%      G5;
%      G6]
%  
%  errorCheck = norm(G, inf)
%  
%  zero_vector = zeros(4, 1);
%  zero_matrix = zeros(4, 4);
%  Id = eye(4);
%  
%  Col1 = [DSS_theta;
%        zero_vector;
%        zero_vector;
%        zero_vector;
%        zero_vector;
%        zero_vector];
%    
% Col2 = [zero_vector;
%         zero_vector;
%         zero_vector;
%         zero_vector;
%         zero_vector;
%          -DSP_sigma];
% 
% Col3 = [zero_vector;
%         zero_vector;
%         zero_vector;
%            FPhi_Rx3;
%          zero_vector;
%          zero_vector];
%      
% Col4 = [zero_vector;
%         zero_vector;
%         zero_vector;
%   partial_alpha_phi;
%         zero_vector;
%         zero_vector];
%     
% 
% size(Dphi_Sx1)    
%     
% Col5 = [      -Id;
%          Dphi_Sx1(1:4, 1:4);
%       zero_matrix;
%       zero_matrix;
%       zero_matrix;
%       zero_matrix];
%   
% Col6 = [zero_matrix;
%                 -Id;
%             DTSR_x2;
%         zero_matrix;
%         zero_matrix;
%         zero_matrix];
%     
% 
% Col7 = [zero_matrix;
%         zero_matrix;
%                 -Id;
%  Dphi_Rx3(1:4, 1:4); 
%         zero_matrix;
%         zero_matrix];
%     
% Col8 = [zero_matrix;
%         zero_matrix;
%         zero_matrix;
%                 -Id;
%             DTRP_x4;
%         zero_matrix];
%     
% Col9 = [zero_matrix;
%         zero_matrix;
%         zero_matrix;
%         zero_matrix;
%                 -Id;
%            Dphi_Px5(1:4, 1:4)];        
%     
% DG = [Col1, Col2, Col3, Col4, Col5, Col6, Col7, Col8, Col9];       
%      
% end    
% 
% 
% 

















