function [P_sigma, DP_sigma] = parmBD_L4_saddleFocus_intval(...
                sigma, R, A, B, C, D, N, tailBound, r_star);
               
            
if isfloat(sigma) == 1
   sigma = intval(sigma);
end
 

z = R*(cos(sigma) + 1i*sin(sigma));
zBar = R*(cos(sigma) - 1i*sin(sigma));
P_sigma = real(evaluate_Parm_intval(z, zBar, A, B, C, D, N));           

B_sigma = sigma + infsup(-r_star, r_star);
Bz = R*(cos(B_sigma) + 1i*sin(B_sigma));
BzBar = R*(cos(B_sigma) - 1i*sin(B_sigma));
DP = evaluate_jacob_Parm_intval(Bz, BzBar, A, B, C, D, N);


%Cauchy estimate: need 
%
% nu' = nu e^{tau}
%
%where tau neasures the amount of domain given up.  In this 
%calculation nu = 1, and nu' = R.  So
%
%   R + r_* = e^{-tau} 
%
%or tau = -log(R+r_*)
%
%Then 
%
% sup_{z \in D_R} \| D P^\infty \| = (2/tau)*tailBound
%
%The factor of 2 is due to the fact that DP has two columns, and we use
%the norm on columns.
%
%
%Now to take the derivative with respect to sigma we apply the chain 
%rule. Since 
%
% D_sigma P(sigma) = D_z P(Rcos(sigma), Rsin(sigma)) D_sigma [Rcos, Rsin]
%                  = Rcos D_z P + Rsin D_z P                  
%
%we have
%
% \| D_sigma P^\infty \| <= 2R \| DP^\infty \|
%                 <= (4R/tau)*tailBound
%


P_sigma = P_sigma + ...
    (R^(N+1))*infsup(-tailBound, tailBound)*intval(ones(4,1));

tau = -log(R+r_star);


DP_sigma = real(DP(:, 1)*(-R*sin(B_sigma)+R*1i*cos(B_sigma)) ...
         + DP(:, 2)*(-R*sin(B_sigma)-R*1i*cos(B_sigma))) + ...
         + R*(4/tau)*infsup(-tailBound, tailBound)*intval(ones(4, 1));
     
     
end
     
     
     
     
     
               