function [xN, uN, yN, vN, R, Omega_x, Omega_y] = ...
    TaylorIntegrator_crtbp_regularized_primary_intval(IC, mu, C, N)

%Initialize coefficient containers:
xN = intval(zeros(N+1, 1));
uN = intval(zeros(N+1, 1));
yN = intval(zeros(N+1, 1));
vN = intval(zeros(N+1, 1));

xN(1) = IC(1);
uN(1) = IC(2);
yN(1) = IC(3);
vN(1) = IC(4);

xN_squared = intval(zeros(N+1, 1));
yN_squared = intval(zeros(N+1, 1));
xN2_plus_yN2_squared  = intval(zeros(N+1, 1));
xN3 = intval(zeros(N+1, 1));
yN3 = intval(zeros(N+1, 1));
xNyN2 = intval(zeros(N+1, 1));
xN2yN = intval(zeros(N+1, 1));

S11 = intval(zeros(N+1, 1));
S12 = intval(zeros(N+1, 1));
S13 = intval(zeros(N+1, 1));

S21 = intval(zeros(N+1, 1));
S22 = intval(zeros(N+1, 1));
S23 = intval(zeros(N+1, 1));

Q = intval(zeros(N+1, 1));  
R = intval(zeros(N+1, 1));
alpha = -intval('3')/intval('2');

Omega_x = intval(zeros(N+1, 1));
Omega_y = intval(zeros(N+1, 1));

xN_squared(1) = xN(1)*xN(1);
yN_squared(1) = yN(1)*yN(1);
xN2_plus_yN2_squared(1)  = ...
    (xN_squared(1) + yN_squared(1))^2;
xN3(1) = xN_squared(1)*xN(1);
yN3(1) = yN_squared(1)*yN(1);
xNyN2(1) = xN(1)*yN_squared(1);
xN2yN(1) = xN_squared(1)*yN(1);

S11(1) = (xN_squared(1) + yN_squared(1))*vN(1);
S12(1) = xN(1)*xN2_plus_yN2_squared(1);

S21(1) = (xN_squared(1) + yN_squared(1))*uN(1);
S22(1) = yN(1)*xN2_plus_yN2_squared(1);


Q(1) = xN2_plus_yN2_squared(1) + 2*(xN_squared(1)-yN_squared(1)) + 1;


R(1) = Q(1)^alpha;
 
S13(1) = (xN3(1) - 3*xNyN2(1) + xN(1))*R(1);
S23(1) = (-yN3(1) + 3*xN2yN(1) + yN(1))*R(1);

Omega_x(1) = 8*S11(1) + 12*S12(1) + 16*mu*xN3(1) + ...
    4*(mu-C)*xN(1) + 8*mu*S13(1);
         
Omega_y(1) = -8*S21(1) + 12*S22(1) - 16*mu*yN3(1) + ...
    4*(mu-C)*yN(1) + 8*mu*S23(1);

xN(2) = uN(1);
uN(2) = Omega_x(1);
yN(2) = vN(1);
vN(2) = Omega_y(1);

%Second order data:
xN_squared(2) = 2*xN(1)*xN(2);
yN_squared(2) = 2*yN(1)*yN(2);
xN2_plus_yN2_squared(2)  = ...
    2*(xN_squared(1) + yN_squared(1))*(xN_squared(2) + yN_squared(2));
xN3(2) = xN_squared(2)*xN(1) + xN_squared(1)*xN(2);
yN3(2) = yN_squared(2)*yN(1) + yN_squared(1)*yN(2);
xNyN2(2) = xN(1)*yN_squared(2) + xN(2)*yN_squared(1);
xN2yN(2) = xN_squared(1)*yN(2) + xN_squared(2)*yN(1);

S11(2) = (xN_squared(1) + yN_squared(1))*vN(2) + ...
    (xN_squared(2) + yN_squared(2))*vN(1);
S12(2) = xN(2)*xN2_plus_yN2_squared(1) + xN(1)*xN2_plus_yN2_squared(2);

S21(2) = (xN_squared(1) + yN_squared(1))*uN(2) + ...
    (xN_squared(2) + yN_squared(2))*uN(1);
S22(2) = yN(1)*xN2_plus_yN2_squared(2) + ...
    yN(2)*xN2_plus_yN2_squared(1);

Q(2) = xN2_plus_yN2_squared(2) + 2*(xN_squared(2)-yN_squared(2));

R(2) = alpha*(Q(1)^(alpha-1))*Q(2);

 
S13(2) = (xN3(1) - 3*xNyN2(1) + xN(1))*R(2) + ...
    (xN3(2) - 3*xNyN2(2) + xN(2))*R(1);
S23(2) = (-yN3(1) + 3*xN2yN(1) + yN(1))*R(2) + ...
    (-yN3(2) + 3*xN2yN(2) + yN(2))*R(1);

Omega_x(2) = 8*S11(2) + 12*S12(2) + 16*mu*xN3(2) + ...
    4*(mu-C)*xN(2) + 8*mu*S13(2);
         
Omega_y(2) = -8*S21(2) + 12*S22(2) - 16*mu*yN3(2) + ...
    4*(mu-C)*yN(2) + 8*mu*S23(2);


for n = 1:N-1
    
    %Update the vector field:
    xN(n+2) = uN(n+1)/(n+1);
    uN(n+2) = Omega_x(n+1)/(n+1);
    yN(n+2) = vN(n+1)/(n+1);
    vN(n+2) = Omega_y(n+1)/(n+1);
    
    %Set up products for next time: 
    
    temp1 = xN(1:n+2);
    temp2 = flipud(temp1);
    xN_squared(n+2) = temp1'*temp2;
    
    temp1 = yN(1:n+2);
    temp2 = flipud(temp1);
    yN_squared(n+2) = temp1'*temp2;
    
    temp1 = xN_squared(1:n+2) + yN_squared(1:n+2);
    temp2 = flipud(temp1);
    xN2_plus_yN2_squared(n+2) = temp1'*temp2;
    
    temp1 = xN_squared(1:n+2);
    temp2 = xN(1:n+2);
    temp2 = flipud(temp2);
    xN3(n+2) = temp1'*temp2;
    
    temp1 = yN_squared(1:n+2);
    temp2 = yN(1:n+2);
    temp2 = flipud(temp2);
    yN3(n+2) = temp1'*temp2;
    
    temp1 = yN_squared(1:n+2);
    temp2 = xN(1:n+2);
    temp2 = flipud(temp2);
    xNyN2(n+2) = temp1'*temp2;
    
    temp1 = xN_squared(1:n+2);
    temp2 = yN(1:n+2);
    temp2 = flipud(temp2);
    xN2yN(n+2) =  temp1'*temp2;

    temp1 = xN_squared(1:n+2) + yN_squared(1:n+2);
    temp2 = vN(1:n+2);
    temp2 = flipud(temp2);
    S11(n+2) = temp1'*temp2;
    
    temp1 = xN2_plus_yN2_squared(1:n+2);
    temp2 = xN(1:n+2);
    temp2 = flipud(temp2);
    S12(n+2) = temp1'*temp2;

    temp1 = xN_squared(1:n+2) + yN_squared(1:n+2);
    temp2 = uN(1:n+2);
    temp2 = flipud(temp2);
    S21(n+2) =  temp1'*temp2;
    
    temp1 = yN(1:n+2);
    temp2 = xN2_plus_yN2_squared(1:n+2);
    temp2 = flipud(temp2);
    S22(n+2) = temp1'*temp2;

    Q(n+2) = xN2_plus_yN2_squared(n+2) + ...
                2*(xN_squared(n+2) - yN_squared(n+2));
    
    temp1 = flipud(R(2:n+1));
    temp2 = Q(2:n+1);
    temp3 = flipud(Q(2:n+1));
    temp4 = R(2:n+1);
    
    the_ks = 1:n;
    temp = (alpha*temp1.*temp2 - temp3.*temp4).*the_ks';
    sum1 = sum(temp);
    
    coef_11 = alpha*Q(1)^(alpha-1);
    coef_12 = 1/((n+1)*Q(1));
    R(n+2) = coef_11*Q(n+2) + coef_12*sum1;
    
%     temp1 = xN_squared(1:n+2) + yN_squared(1:n+2);
%     temp2 = uN(1:n+2)
%     temp2 = flipud(temp2);
%     S21(n+2) =  temp1.*temp2;
    
    temp1 = xN3(1:n+2) - 3*xNyN2(1:n+2) + xN(1:n+2);
    temp2 = R(1:n+2);
    temp2 = flipud(temp2);
    S13(n+2) =  temp1'*temp2;
    
    temp1 = -yN3(1:n+2) + 3*xN2yN(1:n+2) + yN(1:n+2);
    temp2 = R(1:n+2);
    temp2 = flipud(temp2);
    S23(n+2) = temp1'*temp2;

    Omega_x(n+2) = 8*S11(n+2) + 12*S12(n+2) + 16*mu*xN3(n+2) + ...
        4*(mu-C)*xN(n+2) + 8*mu*S13(n+2);
         
    Omega_y(n+2) = -8*S21(n+2) + 12*S22(n+2) - 16*mu*yN3(n+2) + ...
      4*(mu-C)*yN(n+2) + 8*mu*S23(n+2);
    
end


return
