function [DFN, r_DFN] = ...
    evaluate_crtbp_regularized_primary_Differential_poly_Intval(...
                                   xN, uN, yN, vN, ...
                                   r1, r2, r3, r4, ...
                                   mu, C, nu, N);
                                  
alpha2 = -intval(5)/intval(2);    
   
xN_tilde1 = xN;
xN_tilde1(1) = xN(1) - mu;
   
xN_tilde2 = xN;
xN_tilde2(1) = xN(1) + 1 - mu;


[xN1xN1, r_x1x1] = ...
    cauchyProduct_intval(xN_tilde1, xN_tilde1, r1, r1, nu, N);

[xN2xN2, r_x2x2] = ...
    cauchyProduct_intval(xN_tilde2, xN_tilde2, r1, r1, nu, N);

[yNyN, r_yy] = ...
    cauchyProduct_intval(yN, yN, r3, r3, nu, N);


[xN1yN, r_x1y] = ...
    cauchyProduct_intval(xN_tilde1, yN, r1, r3, nu, N);

[xN2yN, r_x2y] = ...
    cauchyProduct_intval(xN_tilde2, yN, r1, r3, nu, N); 
 

%Omega_11 numerators:
Omega_11_numerator1 = -2*xN1xN1 + yNyN;
r_Omega_11_1 = 2*r_x1x1 + r_yy;
Omega_11_numerator2 = -2*xN2xN2 + yNyN;
r_Omega_11_2 = 2*r_x2x2 + r_yy;

%Omega_22_numerators:
Omega_22_numerator1 = xN1xN1 - 2*yNyN;
r_Omega_22_1 = r_x1x1 + 2*r_yy;
Omega_22_numerator2 = xN2xN2 - 2*yNyN;
r_Omega_22_2 = r_x2x2 + 2*r_yy;

%Quotients:
Q1 = xN1xN1 + yNyN;
r_Q1 = r_x1x1 + r_yy;

Q2 = xN2xN2 + yNyN;
r_Q2 = r_x2x2 + r_yy;
                                                           
[T1, r_T1] = cauchyAlphaPower_intval(Q1, alpha2, r_Q1, nu, N);
[T2, r_T2] = cauchyAlphaPower_intval(Q2, alpha2, r_Q2, nu, N);
  

%Build Omega_11:   
[Omega_11_term1, r_Omega_11_term1] = cauchyProduct_intval(...
              Omega_11_numerator1, T1, r_Omega_11_1, r_T1, nu, N);    

 [Omega_11_term2, r_Omega_11_term2] = cauchyProduct_intval(...
              Omega_11_numerator2, T2, r_Omega_11_2, r_T2, nu, N); 
          
 
Omega_11 = -(1-mu)*Omega_11_term1 - mu*Omega_11_term2;
Omega_11(1) = Omega_11(1) + 1;
r_Omega_11 = (1-mu)*r_Omega_11_term1 + mu*r_Omega_11_term2;
        
       
%Build Omega_12:
[Omega_12_term1, r_Omega_12_term1] = cauchyProduct_intval(...
              xN1yN, T1, r_x1y, r_T1, nu, N);    

 [Omega_12_term2, r_Omega_12_term2] = cauchyProduct_intval(...
              xN2yN, T2, r_x2y, r_T2, nu, N); 
          
 
Omega_12 = 3*(1-mu)*Omega_12_term1 + 3*mu*Omega_12_term2;
r_Omega_12 = 3*(1-mu)*r_Omega_12_term1 + 3*mu*r_Omega_12_term2;
    


%Build Omega_22:   
[Omega_22_term1, r_Omega_22_term1] = cauchyProduct_intval(...
              Omega_22_numerator1, T1, r_Omega_22_1, r_T1, nu, N);    

 [Omega_22_term2, r_Omega_22_term2] = cauchyProduct_intval(...
              Omega_22_numerator2, T2, r_Omega_22_2, r_T2, nu, N); 
          
 
Omega_22 = -(1-mu)*Omega_22_term1 - mu*Omega_22_term2;
Omega_22(1) = Omega_22(1) + 1;
r_Omega_22 = (1-mu)*r_Omega_22_term1 + mu*r_Omega_22_term2;


%Set up output:

r_mat = [         0, 0,          0, 0;
         r_Omega_11, 0, r_Omega_12, 0;
                  0, 0,          0, 0;
         r_Omega_12, 0, r_Omega_22, 0];
     
r_DFN = norm(r_mat, inf);

DFN = intval(zeros(4, 4, N+1));

for n = 0:N
   An =  [            0, 0,             0, 0;
          Omega_11(n+1), 0, Omega_12(n+1), 0;
                      0, 0,             0, 0;
          Omega_12(n+1), 0, Omega_22(n+1), 0];
      
   DFN(:, :, n+1) = An; 
end

DFN(:, :, 1) = DFN(:, :, 1) + intval([0,  1, 0, 0;
                                      0,  0, 0, 2;
                                      0,  0, 0, 1;
                                      0, -2, 0, 0]);

