function [phi, Dphi, t_S] = time_T_flow_C1_regularized_intval(...
    u, p, v, q, T, mu, C, r_star, N)

%Evaluate Phi at IC:

IC = [u; p; v; q];
nu = 1.01*abs(T);

[xN, uN, yN, vN, R, Omega_x, Omega_y] = ...
    TaylorIntegrator_crtbp_regularized_primary_intval(IC, mu, C, N);

[proofSuccess, r_proof] = ...
              validateTaylorStep_crtbp_regularized_primary_intval(...
                     xN, uN, yN, vN, mu, C, 2*r_star, nu, N);
           
if proofSuccess == 0
    crashNow = bad_proof_1
end
                                
if inf(r_proof) > 2*r_star
    crashNow = bad_rStar
end
                 
phi = evaluateTaylorIntegrator_planar(xN, uN, yN, vN, T, N) + ...
    infsup(-sup(r_proof), sup(r_proof))*intval(ones(4,1));
                 
%Evaluate the differential on the r_star ball about IC:                

IC_bar = IC + infsup(-r_star, r_star)*intval(ones(4, 1));

[xN_bar, uN_bar, yN_bar, vN_bar, R_bar, Omega_x_bar, Omega_y_bar] = ...
    TaylorIntegrator_crtbp_regularized_primary_intval(IC_bar, mu, C, N);

[proofSuccess, rBar_proof] = ...
              validateTaylorStep_crtbp_regularized_primary_intval(...
                     xN_bar, uN_bar, yN_bar, vN_bar, mu, C, 100*r_star, nu, N);

               
t_S = timeFromRegularizedTime_crtbp_intval(...
    xN_bar, yN_bar, rBar_proof, intval(T), N);                 
                           
                 
                 
if proofSuccess == 0
    crashNow = bad_proof_2
end                 
                 
if inf(rBar_proof) > 100*r_star
    crashNow = bad_rStar_bar
end
                 
                
[DphiN_bar, DFN_bar, r_DFN_bar] = ...
    TaylorIntegrator_crtbp_regularized_primary_VarEq_intval(...
                xN_bar, uN_bar, yN_bar, vN_bar, rBar_proof, mu, C, nu, N);

[proofSuccess, rBar_proof2] = ...
     validateTaylorStep_crtbp_regularized_primary_VarEq_intval(...
                               xN_bar, uN_bar, yN_bar, vN_bar, ...
                               DphiN_bar, DFN_bar, r_DFN_bar, ...
                               mu, C, nu, N);

if proofSuccess == 0
    crashNow = bad_proof_1
end                           
                           
T_bar = T + infsup(-r_star, r_star);                         
Dphi =  evaluateTaylorDifferential_planar(DphiN_bar, T_bar, N) + ...
    infsup(-sup(rBar_proof2), sup(rBar_proof2))*intval(ones(4,4));
 
end






