function [phi, Dphi, partial_T_phi, partial_alpha_phi] = ...
    time_T_flow_PCRTBP_unfolding_intval(x,p,y,q,T,alpha,mu,r_star,N)
       
%Evaluate Phi at IC:

IC = [x; p; y; q];
nu = 1.01*abs(T);

[xN, uN, yN, vN, R1, R2, Omega_x, Omega_y] = ...
    TaylorIntegrator_CRTBP_unfolding_intval(IC, mu, alpha, N);

[proofSuccess, r_proof] = ...
              validateTaylorStep_crtbpUnfolding_intval(...
                     xN, uN, yN, vN, mu, alpha, 2*r_star, nu, N);

if proofSuccess == 0
    crashNow = bad_proof_1
end
                                
% if inf(r_proof) > 2*r_star
%     crashNow = bad_rStar
% end

               
phi = evaluateTaylorIntegrator_planar(xN, uN, yN, vN, T, N) + ...
    infsup(-sup(r_proof), sup(r_proof))*intval(ones(4,1));


%Evaluate the differential on the r_star ball about IC:                
IC_bar = IC + infsup(-r_star, r_star)*intval(ones(4, 1));
alpha_bar = infsup(-r_star, r_star);

[xN_bar, uN_bar, ...
    yN_bar, vN_bar, ...
    R1_bar, R2_bar, ...
    Omega_x_bar, ...
    Omega_y_bar] = ...
      TaylorIntegrator_CRTBP_unfolding_intval(IC_bar, mu, alpha_bar, N);


[proofSuccess, rBar_proof] = ...
              validateTaylorStep_crtbpUnfolding_intval(...
                     xN_bar, uN_bar, yN_bar, vN_bar, ...
                     mu, alpha_bar, 100*r_star, nu, N);


if proofSuccess == 0
    crashNow = proofFailed
end       

%                  
% if inf(rBar_proof) > 2*r_star
%     crashNow = bad_rStar_bar
% end
 
T_bar = T + infsup(-r_star, r_star);

phi_bar = evaluateTaylorIntegrator_planar(...
    xN_bar, uN_bar, yN_bar, vN_bar, T_bar, N) + ...
    infsup(-sup(r_proof), sup(r_proof))*intval(ones(4,1));

partial_T_phi = crtbField_unfolding(phi_bar, mu, alpha_bar);

[DphiN_bar, DFN_bar, r_DFN] = ...
    TaylorIntegrator_crtbpVarEq_unfolding_intval(...
                xN_bar, uN_bar, yN_bar, vN_bar, ...
                rBar_proof, mu, nu, alpha_bar, N);

[proofSuccess, rBar_proof2] = ...
                       validateTaylorStep_crtbpUnfoldingVarEq_intval(...
                              xN_bar, uN_bar, yN_bar, vN_bar, ...
                              DphiN_bar, DFN_bar, r_DFN, alpha_bar, nu, N);                          
                           
if proofSuccess == 0
    crashNow = bad_proof_1
end                           
                                                         
T_bar = T + infsup(-r_star, r_star);                         
Dphi =  evaluateTaylorDifferential_planar(DphiN_bar, T_bar, N) + ...
    infsup(-sup(rBar_proof2), sup(rBar_proof2))*intval(ones(4,4));
                            
                
%Compute and validate the parameter partial:

partial_alpha_phiN = ...
    TaylorIntegrator_crtbp_unfolding_parmPartial_intval(...
        uN_bar, vN_bar, DFN_bar, N);                          
                           
[proofSuccess, r_proof3] = ...
              validateTaylorStep_crtbpUnfoldingParmVarEq_intval(...
                               xN_bar, uN_bar, yN_bar, vN_bar, r_proof, ...
                               DFN_bar, r_DFN, ...
                               partial_alpha_phiN, ...
                               alpha_bar, nu, N);                           
                                       
partial_alpha_phi = evaluateTaylorIntegrator_planar(...
    xN_bar, uN_bar, yN_bar, vN_bar, T_bar, N) + ...
    infsup(-sup(r_proof3), sup(r_proof3))*intval(ones(4,1));                           
                           
                           
end