clear variables
format long


%=======================================================================
%This code is associated with the paper 
%
%Computer assisted proofs for transverse collision and near collision
%orbits in the restricted three body problems
%
%By: Maciej Capinski, Shane Kepley, and J.D. Mireles James
%
%All references to ``the paper'' refer to the title above
%
%
%======================================================================


%This code reproduces the proof of Theorem 1 from the paper.
%
%The parameters are:

mu = 1/4
m1 = 1-mu    
m2 = mu

C = 3.2

%Then we compute the radi for the collision circles in 
% the m2 and m1 regularizations respectively:
%
R0 = sqrt(2*crtbp_regularized_U(0, 0, C, mu))
R1 = sqrt(2*crtbp_regularized_U_m1(0, 0, C, mu))

%Integration time in regularized coordinates
tau = 0.35

%Unfolding parameter:
alpha = 0.0

theta = 2.945584780500718
x1 = [           0.0;
  -1.387134030283961;
                 0.0;
   0.275425456390971]

x2 = [-0.444581369966432;
  -1.038375926396089;
   0.112026231721143;
   0.449167625710804]

x3 = [-0.564897282072410;
   0.978399619177281;
  -0.099609551141525;
  -0.751696444982539];
x4 =  [-0.244097430449604;
   0.878139982728140;
  -0.025435855606098;
   0.543608549989373];
x5 = [0.018086991443588;
  -0.732714475912914;
  -0.703153304556755;
   1.254598547822045]
sigma = 1.459760691418491
T = 2.051635871465205  




%Generate the initial data for the extended multiple
%shooring operator


   %Orbit in Secondary coordinates:
    K1 = 50
    tspan = linspace(0, tau, K1+1);
    IC = x1;

    %Integraton:
    options=odeset('RelTol',1e-13,'AbsTol',1e-13);
    [t, orbit_S] = ode45('crtb_regularizedField_mu1', ...
                           tspan, IC, options, flag, 1-mu, C); 
    
     X = orbit_S';
    
   %Orbit in Rotating (original) coordinates:
    K2 = 200
    IC = x3;
    tspan = linspace(0, T, K2+1);
    %Integraton:

    options=odeset('RelTol',1e-13,'AbsTol',1e-13);
    [t, orbit_R] = ode15s('crtb_Field', ...
         tspan, IC, options, flag, m1, m2);
    
     Y = orbit_R';
     
    %Orbit in Primary coordinates:
     K3 = 55
     tspan = linspace(0, tau, K3+1);
     IC = x5;

    %Integraton:
     options=odeset('RelTol',1e-13,'AbsTol',1e-13);
     [t, orbit_P] = ode45('crtb_regularizedField', ...
                           tspan, IC, options, flag, mu, C); 
    
     Z = orbit_P(1:K3, :)';
      
%Connecting orbit multiple shooting:     
     


     
[G, DG] = primaryToSecondary_collisionMap_multShooting(...
    theta, sigma, T, alpha, X, Y, Z, K1, K2, K3, mu, C, tau);


for k = 1:5
    
    errorNow = norm(G, inf)
    
    State1 = reshape(X, [4*(K1+1), 1]);
    State2 = reshape(Y, [4*(K2+1), 1]);
    State3 = reshape(Z, [4*K3, 1]);
    
    State = [theta;
             sigma;
                 T;
             alpha;
            State1;
            State2;
            State3];
            
   State = State - DG\G;
   
   theta = State(1);
   State(1) = [];
   
   sigma = State(1);
   State(1) = [];
   
   T = State(1);
   State(1) = [];
   
   alpha = State(1);
   State(1) = [];
   
   X = State(1:4*(K1+1));
   X = reshape(X, [4, K1+1]);
   State(1:4*(K1+1)) = [];
   
   Y = State(1:4*(K2+1));
   Y = reshape(Y, [4, K2+1]);
   State(1:4*(K2+1)) = [];
   
   Z = State(1:4*K3);
   Z = reshape(Z, [4, K3]);
   State(1:4*K3) = [];
   
   [G, DG] = primaryToSecondary_collisionMap_multShooting(...
        theta, sigma, T, alpha, X, Y, Z, K1, K2, K3, mu, C, tau);

   errorNow = norm(G, inf)
     
end
    
errorNow = norm(DG\G, inf)
 


%Validate:
theta_int = intval(theta)
sigma_int = intval(sigma)
T_int = intval(T)
alpha_int = intval(0.0);

X_int = intval(X);
Y_int = intval(Y);
Z_int = intval(Z);


C_int = intval('32')/intval('10')
r_star = 10^(-12)    %Should be a float... not interval...
mu_int = intval('1')/intval('4') 
tau_int = intval('35')/intval('100')



tic

N = 15



[G, DG] = primaryToSecondary_collisionMap_multShooting_intval(...
        theta_int, sigma_int, ...
        T_int, alpha_int, ...
        X_int, Y_int, Z_int, ...
        K1, K2, K3, mu_int, ...
        C_int, tau_int, r_star, N);
 
A = inv(mid(DG));    
Y = norm(A*G, inf)

Z = norm(intval(eye(size(A, 1))) - A*DG, inf)    
    
Rad = Y/(1-Z)

proofSuccess = 0;
if sup(Rad) < r_star
    proofSuccess = 1
    errorBound = Rad
end

time = toc



