clear variables
format long


%=======================================================================
%This code is associated with the paper 
%
%Computer assisted proofs for transverse collision and near collision
%orbits in the restricted three body problems
%
%By: Maciej Capinski, Shane Kepley, and J.D. Mireles James
%
%All references to ``the paper'' refer to the title above
%
%
%======================================================================


%This code reproduces the proof of Theorem 2 from the paper.
%


m1 = 0.5    %Copenhagen
m2 = 1-m1
mu = m2

L4 = [0; 0; sqrt(3)/2; 0];

x  = L4(1)
pz = L4(2)
y  = L4(3)
qz = L4(4)

C = Jacobi(L4(1), L4(2), L4(3), L4(4), m1, m2)


[u, p, v, q] = regularCoordinates(x,pz,y, qz, mu)

u
p
v
q

[x_test, x_prime, y_test, y_prime] = ...
    regularCoordinatesInverse(u,p,v,q,mu,C)
x
y
x_test
y_test

%Coordinate Transform not working on x = 0 line!  Come back to this!

C
C_reg = regularizedJacobi(u,p,v,q,mu,C)


C_reg_0 = regularizedJacobi(0,0,0,0,mu,C)
R0 = sqrt(2*crtbp_regularized_U(0, 0, C, mu))

RL4 = constraintFunction_AD(L4, m1, m2)
DF_RL4 = crtb_VectorField_AD_differential(RL4, m1, m2)
F_RL4 = crtb_VectorField_AD(RL4, m1, m2)


[Q, Sigma] = eig(DF_RL4)
eigs = diag(Sigma')


unstableEigsFound = 0;
stableEigsFound = 0;
for k = 1:6
    thisEig = Sigma(k,k);
    eigStability = real(thisEig);
    if unstableEigsFound == 0 & eigStability > 10^(-6)
        lambda_u = thisEig;
        unstableEigsFound = 1;
        unstableIndex = k
        eta_u = Q(:, k);
    elseif unstableEigsFound == 1 & eigStability > 10^(-6)
        lambda_uBar = thisEig;
        unstableIndexBar = k;
        eta_uBar = Q(:, k);
    elseif stableEigsFound == 0 & eigStability < -10^(-6)
        lambda_s = thisEig;
        stableEigsFound = 1;
        stableIndex = k;
        eta_s = Q(:, k);
    elseif stableEigsFound == 1 & eigStability < -10^(-6)
        lambda_sBar = thisEig;
        stableIndexBar = k;
        eta_sBar = Q(:, k);
    end
end


if abs(lambda_uBar - conj(lambda_u)) >= 10^(-14)
    crashNow = badUnstableEig
end

if abs(lambda_sBar - conj(lambda_s)) >= 10^(-14)
    crashNow = badStableEig
end

eigTest1 = norm(DF_RL4*eta_u - lambda_u*eta_u)
eigTest2 = norm(DF_RL4*eta_s - lambda_s*eta_s)

eigTest3 = norm(DF_RL4*eta_uBar - lambda_uBar*eta_uBar)
eigTest4 = norm(DF_RL4*eta_sBar - lambda_sBar*eta_sBar)

if eigTest1 + eigTest2 + eigTest3 + eigTest4 > 10^(-6)
    crashNow = checkEigs
end


%=========================================================
%Parameterization
%=========================================================


%To change order run ``a_hatConvExperiments.m'' with desired N.
load('K_weights_N12.mat')

L4_int = intval(zeros(4, 1));
L4_int(3) = sqrt(intval('3'))/intval('2');

mu_int = intval('1')/intval('2')
m1_int = intval('1') - mu_int
m2_int = mu_int

C_int = Jacobi_intval(...
            L4_int(1), L4_int(2), L4_int(3), L4_int(4), m1_int, m2_int)

RL4_int = constraintFunction_AD_intval(L4_int, m1_int, m2_int);
DF_RL4_int = crtb_VectorField_AD_differential_intval(...
                 RL4_int, m1_int, m2_int);
        
%Validated Eigenvalues:

[lambda_s_int, eta_s_int] = verifyeig(DF_RL4_int, lambda_s, eta_s)
[lambda_sBar_int, eta_sBar_int] = verifyeig(...
    DF_RL4_int, lambda_sBar, eta_sBar)

[lambda_u_int, eta_u_int] = verifyeig(DF_RL4_int, lambda_u, eta_u)
[lambda_uBar_int, eta_uBar_int] = verifyeig(...
    DF_RL4_int, lambda_uBar, eta_uBar)


                                              
                                  
%UnstableManifold
N = 12
scale_u = 0.05

[Au,Bu,Cu,Du,Ru,Su] = computeParameterizationII_CRTBP_intval(...
                            RL4_int, ...
                            lambda_u_int, ...
                            lambda_uBar_int, ...
                            eta_u_int, ...
                            eta_uBar_int, ...
                            intval(1)/intval(2), ...
                            intval(1)/intval(2), ...
                            scale_u, ...
                            N);


  m1_int = intval(1)/intval(2);
  m2_int = intval(1)/intval(2)
                               
[errorBound, proofSuccess] = ...
    validateParmeterizedManifold_intval(...
    Au, Bu, Cu, Du, Ru, Su, L4_int, ...
    lambda_u_int, lambda_uBar_int, ...
    eta_u_int, eta_uBar_int, m1_int, m2_int, N)                        
                        
                        
tailBound = errorBound                        
   
  
                        
numTheta = 50;
numRadii = 10;

                        
unstableMaxs = zeros(1, N+1);
for order = 0:N
    order
    orderMax = 10^(-16);
    for subOrder = 0:order
        m = order - subOrder;
        n = subOrder;
        coeffVect = [mid(Au(m+1, n+1));
                     mid(Bu(m+1, n+1));
                     mid(Cu(m+1, n+1));
                     mid(Du(m+1, n+1))];
        thisMax = max(abs(coeffVect));
        
        if thisMax > orderMax
            orderMax = thisMax;
        end
    end
    unstableMaxs(order + 1) = orderMax;
end
                        
figure
hold on 
plot(log(unstableMaxs)/log(10), 'b*')
                             
                        
                                                                     
numTheta = 250;
numRadii = 10;

Wu_loc = plotManitold_saddleFocusCase(mid(Au),mid(Bu),mid(Cu),mid(Du),...
                                      numRadii,...
                                      numTheta,...
                                      N);                                  
                                  

R = 0.9                                  
                                  
Wu_locBoundry = manifoldBoundry_saddleFocusCase(...
                                                 mid(Au),...
                                                 mid(Bu),...
                                                 mid(Cu),...
                                                 mid(Du),...
                                                 numTheta, R, ...
                                                 N);                                  
                                  
   
figure                                             
hold on 
%plot3(Wu_loc(1, :), Wu_loc(3, :), Wu_loc(2, :), 'b.')
plot3(Wu_locBoundry(1, :), Wu_locBoundry(3, :), Wu_locBoundry(2, :), 'k*')

plot3(-m1, 0, 0, 'k*')
plot3(m2, 0, 0, 'k*')
plot3(L4(1), L4(3), L4(2), 'b*')
axis([-1 1 -1 1])





%===============================================================
%Multiple Shooting Initial guess:
%==============================================================

tau = -0.5

theta = 1.696671399505921
sigma = 0.329444391446397
T = -7.034349083446014

alpha = 0

x1 =[0.000000000000000;
    -0.251085861283952;
                   0.0;
     1.984176375795079]

x1 = [0.0;
  -0.251085861283952;
                   0;
   1.984176375795079];

x3 = [0.003213450375413;
      0.197716496638869;
     -0.404375730348827;
      0.696149210661807];


  
  %============= Generate additional guess data ===================

   %Orbit in Secondary coordinates:
    K1 = 150
    tspan = linspace(0, tau, K1+1);
    IC = x1;

    %Integraton:
    options=odeset('RelTol',1e-13,'AbsTol',1e-13);
    [t, orbit_S] = ode45('crtb_regularizedField', ...
                           tspan, IC, options, flag, mu, C); 
    
     X = orbit_S';
    
   %Orbit in Rotating (original) coordinates:
    K2 = 500
    IC = x3;
    tspan = linspace(0, T, K2+1);
    %Integraton:

    options=odeset('RelTol',1e-13,'AbsTol',1e-13);
    [t, orbit_R] = ode15s('crtb_Field', ...
         tspan, IC, options, flag, m1, m2);
    
     Y = orbit_R(1:K2, :)';
     
     alpha = 0

[G, DG] = primaryCollisionToL4_connectionMap_multShooting(...
    theta, sigma, T, alpha, X, Y, K1, K2, ...
    mid(Au), mid(Bu), mid(Cu), mid(Du), N, mu, C, R, tau);


Error0 = norm(G, inf)

for k = 1:3
    
    State1 = reshape(X, [4*(K1+1), 1]);
    State2 = reshape(Y, [4*K2, 1]);
    
    State = [theta;
             sigma;
                 T;
             alpha;
            State1;
            State2];
    
    State = State - DG\G;
    
   
   theta = State(1);
   State(1) = [];
   
   sigma = State(1);
   State(1) = [];
   
   T = State(1);
   State(1) = [];
   
   alpha = State(1);
   State(1) = [];
   
   X = State(1:4*(K1+1));
   X = reshape(X, [4, K1+1]);
   State(1:4*(K1+1)) = [];
   
   Y = State(1:4*K2);
   Y = reshape(Y, [4, K2]);
   State(1:4*K2) = [];
   
   [G, DG] = primaryCollisionToL4_connectionMap_multShooting(...
    theta, sigma, T, alpha, X, Y, K1, K2, ...
    mid(Au), mid(Bu), mid(Cu), mid(Du), N, mu, C, R, tau);

   errorNow = norm(G, inf)
    
end


finalError = norm(DG\G, inf)
sizeAlpha = abs(alpha)



%Start the proof! 
theta_int = intval(theta)
sigma_int = intval(sigma)
T_int = intval(T)
alpha_int = intval('0.0');

X_int = intval(X);
Y_int = intval(Y);

C_int = intval('3')
r_star = 5*10^(-9)                      %Should be a float... not interval...
mu_int = intval('1')/intval('2') 
tau_int = -intval('1')/intval('2')

R_int = intval('9')/intval('10');
N_taylorInt = 15
N_parm = N


[G, DG, TP] = ...
    primaryCollisionToL4_connectionMap_multShooting_intval(...
        theta_int, sigma_int, T_int, alpha_int, X_int, Y_int, K1, K2, ...
        Au, Bu, Cu, Du, N_parm, tailBound, ...
        mu_int, C_int, R_int, tau_int, r_star, N_taylorInt);



A = inv(mid(DG));    
Y = norm(A*G, inf)

Z = norm(intval(eye(size(A, 1))) - A*DG, inf)    
    
Rad = Y/(1-Z)

proofSuccess = 0;
if sup(Rad) < r_star
    proofSuccess = 1
    errorBound = Rad
end

    
  

C = 3


[X_grid,Y_grid]=meshgrid(-2.2:0.01:2.2);
    R1_grid=((X_grid - m2).^2 + Y_grid.^2).^(1/2);
    R2_grid=((X_grid + m1).^2 + Y_grid.^2).^(1/2);
    Z_grid=[m1*(0.5*R1_grid.^2 + R1_grid.^(-1))+...
        m2*(0.5*R2_grid.^2 + R2_grid.^(-1))];
   
 
    
regOrbitBack =  returnToCRTBP_xy(orbit_S, mu, C);   
   
tspan = linspace(0, -10, 1000);
[P_sigma, DP_sigma] = parmBD_L4_saddleFocus(...
         mid(sigma), R, mid(Au), mid(Bu), mid(Cu), mid(Du), N_parm);


    %Integraton:
    options=odeset('RelTol',1e-13,'AbsTol',1e-13);
    [t,manifoldSegment] = ode15s('crtb_Field', ...
         tspan, P_sigma, options, flag, m1, m2);                
                      

figure
hold on
plot3(-m1, 0, 0, 'k*')
plot3(m2, 0, 0, 'k*')
plot(L4(1), L4(3), 'r*')
plot(orbit_R(:, 1), orbit_R(:, 3), 'b', 'LineWidth', 3)
plot(regOrbitBack(1, :)', regOrbitBack(2, :)', 'LineWidth', 3, ...
    'Color', 'r')
plot3(Wu_locBoundry(1, :), Wu_locBoundry(3, :), Wu_locBoundry(2, :), ...
    'LineWidth', 3, 'Color', 'g')
set(gca, 'FontSize', 18)
axis([-0.6 0.6 -0.8 1.2])

figure
hold on
plot3(-m1, 0, 0, 'k*')
plot3(m2, 0, 0, 'k*')
plot(L4(1), L4(3), 'r*')
plot(orbit_R(:, 1), orbit_R(:, 3), 'b', 'LineWidth', 3)
plot(regOrbitBack(1, :)', regOrbitBack(2, :)', 'LineWidth', 3, ...
    'Color', 'g')
plot(manifoldSegment(:, 1), manifoldSegment(:, 3), 'r.')
axis([-1 1 -1 1])







    
