%homoclinic Newton with unfolding parameter.

clear variables
format long

m1 = 0.5    %Copenhagen
m2 = 1-m1
mu = m2

L4 = [0; 0; sqrt(3)/2; 0];


x  = L4(1)
pz = L4(2)
y  = L4(3)
qz = L4(4)

C = Jacobi(L4(1), L4(2), L4(3), L4(4),m1, m2)



[u, p, v, q] = regularCoordinates(x,pz,y, qz, mu)

u
p
v
q

[x_test, x_prime, y_test, y_prime] = ...
    regularCoordinatesInverse(u,p,v,q,mu,C)
x
y
x_test
y_test

%Coordinate Transform not working on x = 0 line!  Come back to this!

C
C_reg = regularizedJacobi(u,p,v,q,mu,C)


C_reg_0 = regularizedJacobi(0,0,0,0,mu,C)
R0 = sqrt(2*crtbp_regularized_U(0, 0, C, mu))

RL4 = constraintFunction_AD(L4, m1, m2)
DF_RL4 = crtb_VectorField_AD_differential(RL4, m1, m2)
F_RL4 = crtb_VectorField_AD(RL4, m1, m2)


[Q, Sigma] = eig(DF_RL4)
eigs = diag(Sigma')


unstableEigsFound = 0;
stableEigsFound = 0;
for k = 1:6
    thisEig = Sigma(k,k);
    eigStability = real(thisEig);
    if unstableEigsFound == 0 & eigStability > 10^(-6)
        lambda_u = thisEig;
        unstableEigsFound = 1;
        unstableIndex = k
        eta_u = Q(:, k);
    elseif unstableEigsFound == 1 & eigStability > 10^(-6)
        lambda_uBar = thisEig;
        unstableIndexBar = k;
        eta_uBar = Q(:, k);
    elseif stableEigsFound == 0 & eigStability < -10^(-6)
        lambda_s = thisEig;
        stableEigsFound = 1;
        stableIndex = k;
        eta_s = Q(:, k);
    elseif stableEigsFound == 1 & eigStability < -10^(-6)
        lambda_sBar = thisEig;
        stableIndexBar = k;
        eta_sBar = Q(:, k);
    end
end

if abs(lambda_uBar - conj(lambda_u)) >= 10^(-14)
    crashNow = badUnstableEig
end

if abs(lambda_sBar - conj(lambda_s)) >= 10^(-14)
    crashNow = badStableEig
end

eigTest1 = norm(DF_RL4*eta_u - lambda_u*eta_u)
eigTest2 = norm(DF_RL4*eta_s - lambda_s*eta_s)

eigTest3 = norm(DF_RL4*eta_uBar - lambda_uBar*eta_uBar)
eigTest4 = norm(DF_RL4*eta_sBar - lambda_sBar*eta_sBar)

if eigTest1 + eigTest2 + eigTest3 + eigTest4 > 10^(-6)
    crashNow = checkEigs
end


%=========================================================
%Parameterization
%=========================================================


%StableManifold
N = 12
scale_s = 0.05

[As,Bs,Cs,Ds,Rs,Ss] = computeParameterization_CRTBP(...
                            RL4, lambda_s, lambda_sBar, ...
                            eta_s, eta_sBar, m1, m2, ...
                            scale_s, N);

numTheta = 50;
numRadii = 10;



 
%UnstableManifold
N = 12
scale_u = 0.05

[Au,Bu,Cu,Du,Ru,Su] = computeParameterization_CRTBP(...
                            RL4, lambda_u, lambda_uBar, ...
                            eta_u, eta_uBar, m1, m2, ...
                            scale_u, N);

numTheta = 50;
R = 0.9                                  
                                  
Wu_locBoundry = manifoldBoundry_saddleFocusCase(...
                                                 Au,...
                                                 Bu,...
                                                 Cu,...
                                                 Du,...
                                                 numTheta, ...
                                                 R, ...
                                                 N);   

                                             
   
R = 0.9                                  
                                  
Ws_locBoundry = manifoldBoundry_saddleFocusCase(...
                                                 As,...
                                                 Bs,...
                                                 Cs,...
                                                 Ds,...
                                                 numTheta, ...
                                                 R, ...
                                                 N);                                                
                                      

%                                              
%                                              
% 
%  sigma = 0.39468
%  theta = 5.65487
%  T0 = 12.225
%  alpha0 = 0                                             
%                                              
%                                              
%  
% [G, DG] = ...
%     homoclinic_as_zero_map(sigma, theta, T0, alpha0, R, ...
%                            Au, Bu, Cu, Du, ...
%                            As, Bs, Cs, Ds, ...
%                            N, mu);
%   
%  
%  error0 = norm(inv(DG)*G, inf)
%  
%  v_now = [sigma; theta; T0; alpha0];
%  
%  for k = 1:8
%     
%      [G, DG] = ...
%     homoclinic_as_zero_map(v_now(1), v_now(2), v_now(3), v_now(4), R, ...
%                            Au, Bu, Cu, Du, ...
%                            As, Bs, Cs, Ds, ...
%                            N, mu);
%      
%      thisError = norm(inv(DG)*G, inf)  
%      
%      delta = -DG\G;                  
%      v_now = v_now + delta;                  
%      
%  end
%  
%  sigma_bar = v_now(1)
%  theta_bar = v_now(2)
%  T_bar = v_now(3)
%  alpha_bar = v_now(4)
%   
%  
%  return
 

%Initial Guess Data:

sigma_bar = 0.394684948615285
theta_bar = 5.888500360027008
T_bar = 12.300730813113956
alpha_bar = 0.0
   

%Generate multiple shooting initial data:

 sigma1 = R*cos(sigma_bar);
 sigma2 = R*sin(sigma_bar);
 P_sigma = evaluate_complexCase(sigma1, sigma2, Au, Bu, Cu, Du, N);
 
 tspan = linspace(0, T_bar, 5000);
 IC = P_sigma;

    %Integraton:
    options=odeset('RelTol',1e-13,'AbsTol',1e-13);
    [t, orbit1] = ode15s('crtb_Field', ...
    tspan, IC, options, flag, m1, m2);                                  
                                  
                                                                                                    
figure 
hold on 
plot3(Wu_locBoundry(1, :), Wu_locBoundry(3, :), Wu_locBoundry(2, :), 'b.')
plot3(Ws_locBoundry(1, :), Ws_locBoundry(3, :), Ws_locBoundry(2, :), 'r.')      
plot3(orbit1(:, 1), orbit1(:, 3), orbit1(:, 2), 'k', 'LineWidth', 3)

omega1 = R*cos(theta_bar);
omega2 = R*sin(theta_bar);
Q_omega = evaluate_complexCase(omega1, omega2, As, Bs, Cs, Ds, N);
plot3(Q_omega(1), Q_omega(3), Q_omega(2), 'ko')
plot3(P_sigma(1), P_sigma(3), P_sigma(2), 'k*')

plot3(-m1, 0, 0, 'k*')
plot3(m2, 0, 0, 'k*')
plot3(L4(1), L4(3), L4(2), 'b*')


%========================= Multiple Shooting ==============================



                        
                        
K = 600
IC = P_sigma;
T = T_bar
tspan = linspace(0, (K*T-T)/K, K);
    %Integraton:

    options=odeset('RelTol',1e-13,'AbsTol',1e-13);
    [t, orbit_R] = ode15s('crtb_Field', ...
         tspan, IC, options, flag, m1, m2);
    
X = orbit_R';

sigma = sigma_bar
theta = theta_bar
alpha = 0

 [G, DG] = ...
    homoclinic_multipleShooting_map(sigma, theta, T, alpha, X, K, ...
                           Au, Bu, Cu, Du, ...
                           As, Bs, Cs, Ds, ...
                           N, mu);
                       
                       
Error = norm(G, inf)


for m = 1:5
    
    State = [sigma;
             theta;
             T;
             alpha;
             reshape(X, [4*K, 1])];
    
    State = State - DG\G;
    
    sigma = State(1);
    State(1) = [];
    
    theta = State(1);
    State(1) = [];
    
    T = State(1);
    State(1) = [];
    
    alpha = State(1);
    State(1) = [];
    
    X = State(1:4*K);
    X = reshape(X, [4, K]);
    
    
   [G, DG] = ...
        homoclinic_multipleShooting_map(sigma, theta, T, alpha, X, K, ...
                           Au, Bu, Cu, Du, ...
                           As, Bs, Cs, Ds, ...
                           N, mu);
    
    errorNow = norm(G, inf)
end 



P_test = evaluate_complexCase(R*cos(sigma), R*sin(sigma), ...
                              Au, Bu, Cu, Du, N);
                          
 error1 = norm(P_test - X(:, 1))                         

 
 

%Interval Data: 
sigma_int = intval(sigma)
theta_int = intval(theta)
T_int = intval(T)
alpha_int = intval(0);
X_int = intval(X);

L4_int = [intval(0); 
          intval(0); 
          sqrt(intval(3))/intval(2); 
          intval(0)];


mu_int = intval('1')/intval('2')
m1_int = intval('1') - mu_int
m2_int = mu_int

C_int = Jacobi_intval(...
            L4_int(1), L4_int(2), L4_int(3), L4_int(4), m1_int, m2_int)

              
R_int = intval('9')/intval('10');
N_taylorInt = 12
N_parm = N


%Manifolds:

load('K_weights_N12.mat')
RL4_int = constraintFunction_AD_intval(L4_int, m1_int, m2_int);
DF_RL4_int = crtb_VectorField_AD_differential_intval(...
                 RL4_int, m1_int, m2_int);

%Validated Eigenvalues:

[lambda_s_int, eta_s_int] = verifyeig(DF_RL4_int, lambda_s, eta_s)
[lambda_sBar_int, eta_sBar_int] = verifyeig(...
    DF_RL4_int, lambda_sBar, eta_sBar)

[lambda_u_int, eta_u_int] = verifyeig(DF_RL4_int, lambda_u, eta_u)
[lambda_uBar_int, eta_uBar_int] = verifyeig(...
    DF_RL4_int, lambda_uBar, eta_uBar)             
             
    
N = 12
scale_u = 0.05

[Au_int,Bu_int,Cu_int,Du_int,Ru_int,Su_int] = ...
    computeParameterizationII_CRTBP_intval(...
                            RL4_int, ...
                            lambda_u_int, ...
                            lambda_uBar_int, ...
                            eta_u_int, ...
                            eta_uBar_int, ...
                            m1_int, ...
                            m2_int, ...
                            scale_u, ...
                            N);


                               
[errorBound, proofSuccess] = ...
    validateParmeterizedManifold_intval(...
    Au_int, Bu_int, Cu_int, Du_int, Ru_int, Su_int, L4_int, ...
    lambda_u_int, lambda_uBar_int, ...
    eta_u_int, eta_uBar_int, m1_int, m2_int, N)                        
                        
                        
tailBound_u = errorBound 


scale_s = 0.05

[As_int,Bs_int,Cs_int,Ds_int,Rs_int,Ss_int] = ...
    computeParameterizationII_CRTBP_intval(...
                            RL4_int, ...
                            lambda_s_int, ...
                            lambda_sBar_int, ...
                            eta_s_int, ...
                            eta_sBar_int, ...
                            m1_int, ...
                            m2_int, ...
                            scale_s, ...
                            N);


                               
[errorBound, proofSuccess] = ...
    validateParmeterizedManifold_intval(...
    As_int, Bs_int, Cs_int, Ds_int, Rs_int, Ss_int, L4_int, ...
    lambda_s_int, lambda_sBar_int, ...
    eta_s_int, eta_sBar_int, m1_int, m2_int, N)                        
                        
                        
tailBound_s = errorBound 



[G, DG] = ...
     homoclinic_multipleShooting_map(mid(sigma_int), ...
                           mid(theta_int), ...
                           mid(T_int), ...
                           mid(alpha_int), ...
                           mid(X_int), ...
                           K, ...
                           mid(Au_int), ...
                           mid(Bu_int), ...
                           mid(Cu_int), ...
                           mid(Du_int), ...
                           mid(As_int), ...
                           mid(Bs_int), ...
                           mid(Cs_int), ...
                           mid(Ds_int), ...
                           N, mu);

for m = 1:2
    
    State = [mid(sigma_int);
             mid(theta_int);
             mid(T_int);
             mid(alpha_int);
             reshape(mid(X_int), [4*K, 1])];
    
    State = State - DG\G;
    
    sigma_int = intval(State(1));
    State(1) = [];
    
    theta_int = intval(State(1));
    State(1) = [];
    
    T_int = intval(State(1));
    State(1) = [];
    
    alpha_int = intval(State(1));
    State(1) = [];
    
    X_int = intval(State(1:4*K));
    X_int = reshape(X_int, [4, K]);
    
    
   
[G, DG] = ...
     homoclinic_multipleShooting_map(mid(sigma_int), ...
                           mid(theta_int), ...
                           mid(T_int), ...
                           mid(alpha_int), ...
                           mid(X_int), ...
                           K, ...
                           mid(Au_int), ...
                           mid(Bu_int), ...
                           mid(Cu_int), ...
                           mid(Du_int), ...
                           mid(As_int), ...
                           mid(Bs_int), ...
                           mid(Cs_int), ...
                           mid(Ds_int), ...
                           N, mu);
    
    errorNow = norm(G, inf)
end 



alpha_int = intval(0)


r_star = 5*10^(-9)
N_parm = 12
N_taylor = 12
 

[G, DG] = ...
    homoclinic_multipleShooting_map_intval(...
                           sigma_int, ...
                           theta_int, ...
                           T_int, ...
                           alpha_int, ...
                           X_int, ...
                           R_int, ...
                           K, ...
                           Au_int, Bu_int, Cu_int, Du_int, ...
                           As_int, Bs_int, Cs_int, Ds_int, ...
                           tailBound_u, tailBound_s, N_parm, ...
                           mu_int, N_taylor, r_star);

                       
                       
                   
A = inv(mid(DG));    
Y = norm(A*G, inf)

Z = norm(intval(eye(size(A, 1))) - A*DG, inf)    
    
Rad = Y/(1-Z)

proofSuccess = 0;
if sup(Rad) < r_star
    proofSuccess = 1
    errorBound = Rad
end                       
                       
                       
% 
%         

