%homoclinic Newton with unfolding parameter.

clear variables
format long

m1 = 0.5    %Copenhagen
m2 = 1-m1
mu = m2

L4 = [0; 0; sqrt(3)/2; 0];

x  = L4(1)
pz = L4(2)
y  = L4(3)
qz = L4(4)

C = Jacobi(L4(1), L4(2), L4(3), L4(4),m1, m2)



[u, p, v, q] = regularCoordinates(x,pz,y, qz, mu)

u
p
v
q

[x_test, x_prime, y_test, y_prime] = ...
    regularCoordinatesInverse(u,p,v,q,mu,C)
x
y
x_test
y_test

%Coordinate Transform not working on x = 0 line!  Come back to this!

C
C_reg = regularizedJacobi(u,p,v,q,mu,C)


C_reg_0 = regularizedJacobi(0,0,0,0,mu,C)
R0 = sqrt(2*crtbp_regularized_U(0, 0, C, mu))

RL4 = constraintFunction_AD(L4, m1, m2)
DF_RL4 = crtb_VectorField_AD_differential(RL4, m1, m2)
F_RL4 = crtb_VectorField_AD(RL4, m1, m2)


[Q, Sigma] = eig(DF_RL4)
eigs = diag(Sigma')


unstableEigsFound = 0;
stableEigsFound = 0;
for k = 1:6
    thisEig = Sigma(k,k);
    eigStability = real(thisEig);
    if unstableEigsFound == 0 & eigStability > 10^(-6)
        lambda_u = thisEig;
        unstableEigsFound = 1;
        unstableIndex = k
        eta_u = Q(:, k);
    elseif unstableEigsFound == 1 & eigStability > 10^(-6)
        lambda_uBar = thisEig;
        unstableIndexBar = k;
        eta_uBar = Q(:, k);
    elseif stableEigsFound == 0 & eigStability < -10^(-6)
        lambda_s = thisEig;
        stableEigsFound = 1;
        stableIndex = k;
        eta_s = Q(:, k);
    elseif stableEigsFound == 1 & eigStability < -10^(-6)
        lambda_sBar = thisEig;
        stableIndexBar = k;
        eta_sBar = Q(:, k);
    end
end

if abs(lambda_uBar - conj(lambda_u)) >= 10^(-14)
    crashNow = badUnstableEig
end

if abs(lambda_sBar - conj(lambda_s)) >= 10^(-14)
    crashNow = badStableEig
end

eigTest1 = norm(DF_RL4*eta_u - lambda_u*eta_u)
eigTest2 = norm(DF_RL4*eta_s - lambda_s*eta_s)

eigTest3 = norm(DF_RL4*eta_uBar - lambda_uBar*eta_uBar)
eigTest4 = norm(DF_RL4*eta_sBar - lambda_sBar*eta_sBar)

if eigTest1 + eigTest2 + eigTest3 + eigTest4 > 10^(-6)
    crashNow = checkEigs
end


%=========================================================
%Parameterization
%=========================================================


%StableManifold
N = 12
scale_s = 0.05

[As,Bs,Cs,Ds,Rs,Ss] = computeParameterization_CRTBP(...
                            RL4, lambda_s, lambda_sBar, ...
                            eta_s, eta_sBar, m1, m2, ...
                            scale_s, N);

numTheta = 50;
numRadii = 10;



 
%UnstableManifold
N = 12
scale_u = 0.05

[Au,Bu,Cu,Du,Ru,Su] = computeParameterization_CRTBP(...
                            RL4, lambda_u, lambda_uBar, ...
                            eta_u, eta_uBar, m1, m2, ...
                            scale_u, N);

numTheta = 50;
R = 0.9                                  
                                  
Wu_locBoundry = manifoldBoundry_saddleFocusCase(...
                                                 Au,...
                                                 Bu,...
                                                 Cu,...
                                                 Du,...
                                                 numTheta, ...
                                                 R, ...
                                                 N);   

                                             
   
R = 0.9                                  
                                  
Ws_locBoundry = manifoldBoundry_saddleFocusCase(...
                                                 As,...
                                                 Bs,...
                                                 Cs,...
                                                 Ds,...
                                                 numTheta, ...
                                                 R, ...
                                                 N);                                                
                                      

T1 = 1.8635                                             
 T2 = 5                                             
 
%  
%  sigma = 1.411845518097997
%  theta = 5.988827141583656
%  T = 12.623885197754252
%  alpha0 = 0
% 
%  
%  z1 = R*cos(sigma);
%  z2 = R*sin(sigma);
%  P_sigma = evaluate_complexCase(z1, z2, Au, Bu, Cu, Du, N);
%  
% 
%  w1 = R*cos(theta);
%  w2 = R*sin(theta);
%  Q_theta = evaluate_complexCase(w1, w2, As, Bs, Cs, Ds, N);
%   
%  T = 4.75
%  
%  tspan = linspace(0, T, 5000);
%  
%  %tspan = linspace(0, 12.225, 5000);
%  IC = P_sigma;
% 
%     %Integraton:
%     options=odeset('RelTol',1e-13,'AbsTol',1e-13);
%     [t, orbit1] = ode15s('crtb_Field', ...
%     tspan, IC, options, flag, m1, m2);
%  
% T2 = 5
% 
%  tspan = linspace(0, -T2, 5000);
%  
%  %tspan = linspace(0, 12.225, 5000);
%  IC = Q_theta;
% 
%     %Integraton:
%     options=odeset('RelTol',1e-13,'AbsTol',1e-13);
%     [t, orbit2] = ode15s('crtb_Field', ...
%     tspan, IC, options, flag, m1, m2);
%   
% 
% [TRS, DTRS] = transform_rotatingToSecondary(orbit1(end, :)', mu, C) 
% IC = TRS 
% 
% T1 = 1.8635
% 
%   tspan = linspace(0, T1, 5000);
%     
%     %Integraton:
%     options=odeset('RelTol',1e-13,'AbsTol',1e-13);
%     [t,thisOrbit] = ode45('crtb_regularizedField_mu1', ...
%                            tspan, IC, options, flag, 1-mu, C);                  
%                       
%  
% thisOrbit_back = returnToCRTBP_mu1(thisOrbit, mu, C);                       
%      
% 
% 
% figure 
% hold on 
% plot3(Wu_locBoundry(1, :), Wu_locBoundry(3, :), Wu_locBoundry(2, :), 'b.')
% plot3(Ws_locBoundry(1, :), Ws_locBoundry(3, :), Ws_locBoundry(2, :), 'r.')      
% plot3(orbit1(:, 1), orbit1(:, 3), orbit1(:, 2), 'k', 'LineWidth', 3)
% plot3(orbit2(:, 1), orbit2(:, 3), orbit2(:, 2), 'g', 'LineWidth', 3)
% plot3(thisOrbit_back(1,:), thisOrbit_back(3,:), thisOrbit_back(2,:), 'c', 'LineWidth', 3)
% 
% plot3(-m1, 0, 0, 'k*')
% plot3(m2, 0, 0, 'k*')
% plot3(L4(1), L4(3), L4(2), 'b*')
% 
% 


%
%
%
%
%
%
%
%
% 
% alpha = 0
% x1 = P_sigma
% x2 = orbit1(end, :)'
% x3 = TRS
% x4 = thisOrbit(end, :)'
% x5 = orbit2(end, :)'
% 
% [G, DG] = homoclinic_L4_flyBy_secondary_connectionMap(...
%     sigma, theta, T, alpha, ...
%     x1, x2, x3, x4, x5, ...
%     Au, Bu, Cu, Du, ...
%     As, Bs, Cs, Ds, ...
%     N, mu, C, R, T1, T2);
% 
% error0 = norm(G, inf)
% 
% for n = 1:6
%     
%     state = [sigma;
%              theta;
%                  T;
%              alpha;
%                 x1;
%                 x2;
%                 x3;
%                 x4;
%                 x5];
%             
%    Delta = -DG\G;         
%    state = state + Delta;
%    
%    sigma = state(1);
%    theta = state(2);
%    T = state(3);
%    alpha = state(4);
%    x1 = state(5:8);
%    x2 = state(9:12);
%    x3 = state(13:16);
%    x4 = state(17:20);
%    x5 = state(21:24);
%    
%   [G, DG] = homoclinic_L4_flyBy_secondary_connectionMap(...
%     sigma, theta, T, alpha, ...
%     x1, x2, x3, x4, x5, ...
%     Au, Bu, Cu, Du, ...
%     As, Bs, Cs, Ds, ...
%     N, mu, C, R, T1, T2);
% 
%     thisError = norm(G, inf)
% 
% end
% 
% 
% finalError = norm(DG\G, inf)

sigma = 1.411845518093465
theta = 5.988827141587524
T = 4.753189987600322
alpha = 0.0

x1 = [ -0.037058535596501;
  -0.007623220825047;
   0.873641524117294;
   0.033084516428934];

x2 = [ -0.243792825826104;
  -1.231115812339929;
   0.191555402161713;
  -0.508371512711423];

x3 = [0.536705932004517;
  -1.502936899979675;
   0.178454709310070;
  -0.106295184232356];

x4 = [-0.504618223786019;
  -0.258236028925146;
  -0.463683953250136;
  -1.155517791533997];
  
x5 = [ -0.460363256724718;
  -0.431694929685453;
   0.467966745774326;
   0.748266444559725];



%==========================================================================
%Multiple Shooting: 
%==========================================================================


%Multiple Shooting:


   %Orbit leaving the unstable manifold (rotating coordinates):
    K1 = 200
    tspan = linspace(0, T, K1+1);
    IC = x1;

    %Integraton:
    options=odeset('RelTol',1e-13,'AbsTol',1e-13);
    [t, orbit1] = ode15s('crtb_Field', ...
                     tspan, IC, options, flag, m1, m2);
    
     X = orbit1';
    
   %FlyBy orbit segment (Levi-civita small body coordinates):
    K2 = 300
    IC = x3;
    tspan = linspace(0, T1, K2+1);
    
    %Integraton:
    options=odeset('RelTol',1e-13,'AbsTol',1e-13);
    [t, orbit2] = ode45('crtb_regularizedField_mu1', ...
                           tspan, IC, options, flag, 1-mu, C); 
    
     Y = orbit2';
     
    %Orbit returning to the stable manifold (rotating coordinates):
    K3 = 100
    tspan = linspace(0, T2, K3+1);
    IC = x5;

    %Integraton:
    options=odeset('RelTol',1e-13,'AbsTol',1e-13);
    [t, orbit3] = ode15s('crtb_Field', ...
                     tspan, IC, options, flag, m1, m2);
    
     Z = orbit3(1:K3, :)'; 
     
     alpha = 0



[G, DG] = ...
    homoclinic_L4_flyBy_secondary_connectionMap_multShooting(...
         sigma, theta, T, alpha, ...
         X, Y, Z, ...
         Au, Bu, Cu, Du, ...
         As, Bs, Cs, Ds, ...
         N, K1, K2, K3, ...
         mu, C, R, T1, T2);
     

sizeTest = size(G)     
testError = norm(G)




for k = 1:3
    
    errorNow = norm(G, inf)
    
    State1 = reshape(X, [4*(K1+1), 1]);
    State2 = reshape(Y, [4*(K2+1), 1]);
    State3 = reshape(Z, [4*K3, 1]);
    
    State = [sigma;
             theta;
                 T;
             alpha;
            State1;
            State2;
            State3];
            
   State = State - DG\G;
   
   sigma = State(1);
   State(1) = [];
   
   theta = State(1);
   State(1) = [];
   
   T = State(1);
   State(1) = [];
   
   alpha = State(1);
   State(1) = [];
   
   X = State(1:4*(K1+1));
   X = reshape(X, [4, K1+1]);
   State(1:4*(K1+1)) = [];
   
   Y = State(1:4*(K2+1));
   Y = reshape(Y, [4, K2+1]);
   State(1:4*(K2+1)) = [];
   
   Z = State(1:4*K3);
   Z = reshape(Z, [4, K3]);
   State(1:4*K3) = [];
   
   [G, DG] = ...
    homoclinic_L4_flyBy_secondary_connectionMap_multShooting(...
         sigma, theta, T, alpha, ...
         X, Y, Z, ...
         Au, Bu, Cu, Du, ...
         As, Bs, Cs, Ds, ...
         N, K1, K2, K3, ...
         mu, C, R, T1, T2);
   
     
end
    
errorFinal = norm(DG\G, inf)
 


%==========================================================================
%Start the proof
%==========================================================================


sigma_int = intval(sigma)
theta_int = intval(theta)
T_int = intval(T)
alpha_int = intval(0)
X_int = intval(X);
Y_int = intval(Y);
Z_int = intval(Z);


mu_int = intval('1')/intval('2')
m1_int = intval('1') - mu_int
m2_int = mu_int


L4_int = [intval(0); 
          intval(0); 
          sqrt(intval(3))/intval(2); 
          intval(0)];

C_int = Jacobi_intval(...
            L4_int(1), L4_int(2), L4_int(3), L4_int(4), m1_int, m2_int)

              
R_int = intval('9')/intval('10');
T1_int = intval(T1)
T2_int = intval(T2)
N_taylorInt = 12
N_parm = N


load('K_weights_N12.mat')
RL4_int = constraintFunction_AD_intval(L4_int, m1_int, m2_int);
DF_RL4_int = crtb_VectorField_AD_differential_intval(...
                 RL4_int, m1_int, m2_int);

%Validated Eigenvalues:

[lambda_s_int, eta_s_int] = verifyeig(DF_RL4_int, lambda_s, eta_s)
[lambda_sBar_int, eta_sBar_int] = verifyeig(...
    DF_RL4_int, lambda_sBar, eta_sBar)

[lambda_u_int, eta_u_int] = verifyeig(DF_RL4_int, lambda_u, eta_u)
[lambda_uBar_int, eta_uBar_int] = verifyeig(...
    DF_RL4_int, lambda_uBar, eta_uBar)             
             
    
N = 12
scale_u = 0.05

[Au_int,Bu_int,Cu_int,Du_int,Ru_int,Su_int] = ...
    computeParameterizationII_CRTBP_intval(...
                            RL4_int, ...
                            lambda_u_int, ...
                            lambda_uBar_int, ...
                            eta_u_int, ...
                            eta_uBar_int, ...
                            m1_int, ...
                            m2_int, ...
                            scale_u, ...
                            N_parm);

                             
[errorBound, proofSuccess] = ...
    validateParmeterizedManifold_intval(...
    Au_int, Bu_int, Cu_int, Du_int, Ru_int, Su_int, L4_int, ...
    lambda_u_int, lambda_uBar_int, ...
    eta_u_int, eta_uBar_int, m1_int, m2_int, N_parm)                        
                        
                        
tailBound_u = errorBound 


scale_s = 0.05

[As_int,Bs_int,Cs_int,Ds_int,Rs_int,Ss_int] = ...
    computeParameterizationII_CRTBP_intval(...
                            RL4_int, ...
                            lambda_s_int, ...
                            lambda_sBar_int, ...
                            eta_s_int, ...
                            eta_sBar_int, ...
                            m1_int, ...
                            m2_int, ...
                            scale_s, ...
                            N_parm);


                               
[errorBound, proofSuccess] = ...
    validateParmeterizedManifold_intval(...
    As_int, Bs_int, Cs_int, Ds_int, Rs_int, Ss_int, L4_int, ...
    lambda_s_int, lambda_sBar_int, ...
    eta_s_int, eta_sBar_int, m1_int, m2_int, N_parm)                        
                        
                        
tailBound_s = errorBound 


[G, DG] = ...
    homoclinic_L4_flyBy_secondary_connectionMap_multShooting(...
         mid(sigma_int), ...
         mid(theta_int), ...
         mid(T_int), ...
         mid(alpha_int), ...
         mid(X_int), ...
         mid(Y_int), ...
         mid(Z_int), ...
         mid(Au_int), ...
         mid(Bu_int), ...
         mid(Cu_int), ...
         mid(Du_int), ...
         mid(As_int), ...
         mid(Bs_int), ...
         mid(Cs_int), ...
         mid(Ds_int), ...
         N, K1, K2, K3, ...
         mid(mu_int), ...
         mid(C_int), ...
         mid(R_int), ...
         mid(T1_int), ...
         mid(T2_int));



for k = 1:3
    
    errorNow = norm(G, inf)
    
    State1 = reshape(mid(X_int), [4*(K1+1), 1]);
    State2 = reshape(mid(Y_int), [4*(K2+1), 1]);
    State3 = reshape(mid(Z_int), [4*K3, 1]);
    
    State = [mid(sigma_int);
             mid(theta_int);
             mid(T_int);
             mid(alpha_int);
            State1;
            State2;
            State3];
            
   State = State - DG\G;
   
   sigma_int = intval(State(1));
   State(1) = [];
   
   theta_int = intval(State(1));
   State(1) = [];
   
   T_int = intval(State(1));
   State(1) = [];
   
   alpha_int = intval(State(1));
   State(1) = [];
   
   X_int = intval(State(1:4*(K1+1)));
   X_int = reshape(X_int, [4, K1+1]);
   State(1:4*(K1+1)) = [];
   
   Y_int = intval(State(1:4*(K2+1)));
   Y_int = reshape(Y_int, [4, K2+1]);
   State(1:4*(K2+1)) = [];
   
   Z_int = intval(State(1:4*K3));
   Z_int = reshape(Z_int, [4, K3]);
   State(1:4*K3) = [];
   
   
[G, DG] = ...
    homoclinic_L4_flyBy_secondary_connectionMap_multShooting(...
         mid(sigma_int), ...
         mid(theta_int), ...
         mid(T_int), ...
         mid(alpha_int), ...
         mid(X_int), ...
         mid(Y_int), ...
         mid(Z_int), ...
         mid(Au_int), ...
         mid(Bu_int), ...
         mid(Cu_int), ...
         mid(Du_int), ...
         mid(As_int), ...
         mid(Bs_int), ...
         mid(Cs_int), ...
         mid(Ds_int), ...
         N, K1, K2, K3, ...
         mid(mu_int), ...
         mid(C_int), ...
         mid(R_int), ...
         mid(T1_int), ...
         mid(T2_int));
   
     
end
    
errorFinal = norm(DG\G, inf)
 


%==========================================================================
%Tests:


N_taylor = 12
r_star = 10^(-8)

k = 200
  

    [phi_Rxk, Dphi_Rxk, partial_T_phi_Rxk, partial_alpha_phi_Rxk] = ...
         time_T_flow_PCRTBP_unfolding_intval(...
             X_int(1,k), X_int(2,k), X_int(3,k), X_int(4,k), ...
             T_int/200, alpha_int, mu_int, r_star, N_taylor);  
         
         
errorTest = norm(phi_Rxk - X_int(:, k+1), inf)
radTest = rad(norm(Dphi_Rxk, inf))
   

%==========================================================================


N_taylor = 12
r_star = 10^(-8)

 [G, DG, TS] = ...
    homoclinic_L4_flyBy_secondary_connectionMap_multShooting_intval(...
         sigma_int, theta_int, T_int, alpha_int, ...
         X_int, Y_int, Z_int, N_parm, N_taylor, ...
         Au_int, Bu_int, Cu_int, Du_int, tailBound_u, ...
         As_int, Bs_int, Cs_int, Ds_int, tailBound_s, ...
         K1, K2, K3, ...
         mu_int, C_int, R_int, ...
         T1_int, T2_int, r_star);

 
               
A = inv(mid(DG));    
Y = norm(A*G, inf)

Z = norm(intval(eye(size(A, 1))) - A*DG, inf)    
    
Rad = Y/(1-Z)

proofSuccess = 0;
if sup(Rad) < r_star
    proofSuccess = 1
    errorBound = Rad
end                       
             


T_total_int = T_int + TS + T2_int








